---
title: Running Jobs
#tags:
#keywords:
last_updated: 18 June 2019
#summary: ""
sidebar: merlin6_sidebar
permalink: /merlin6/running-jobs.html
---

## Commands for running jobs

* **``sbatch``**: to submit a batch script to Slurm
  * Use **``squeue``** for checking jobs status 
  * Use **``scancel``** for deleting a job from the queue.
* **``srun``**: to run parallel jobs in the batch system
* **``salloc``**: to obtain a Slurm job allocation (a set of nodes), execute command(s), and then release the allocation when the command is finished.

## Slurm parameters

For a complete list of options and parameters available is recommended to use the **man** pages (``man sbatch``, ``man srun``, ``man salloc``). Please, notice that behaviour for some parameters might change depending on the command (in example, ``--exclusive`` behaviour in ``sbatch`` differs from ``srun``.

In this chapter we show the basic parameters which are usually needed in the Merlin cluster.

### Running in Merlin5 & Merlin6

* For running jobs in the **Merlin6** computing nodes, users have to add the following option:

```bash
#SBATCH --clusters=merlin6
```

* For running jobs in the **Merlin5** computing nodes, users have to add the following options:

```bash
#SBATCH --clusters=merlin5
```

***For advanced users:*** If you do not care where to run the jobs (**Merlin5** or **Merlin6**) you can skip this setting, however you must make sure that your code can run on both clusters without any problem and you have defined proper settings in your *batch* script.

### Partitions

**Merlin6** contains 4 partitions for general purpose, while **Merlin5** contains 1 single CPU partition (for historical reasons):

   * **Merlin6** CPU partitions are 3: ``general``, ``daily`` and ``hourly``.
   * **Merlin6** GPU partition is 1: ``gpu``.
   * **Merlin5** CPU partition is 1: ``merlin``

For Merlin6, if no partition is defined, ``general`` will be the default, while for Merlin5 is ``merlin``. Partitions can be changed by defining the ``--partition`` option as follows:

```bash
#SBATCH --partition=<partition_name>  # Partition to use. 'general' is the 'default' in Merlin6.
```

Please check the section [Slurm Configuration#Merlin6 Slurm Partitions] for more information about Merlin6 partition setup.

### Enabling/disabling hyperthreading

Computing nodes in **merlin6** have hyperthreading enabled: every core is running two threads. It means that for many cases it needs to be disabled and only those multithread-based applications will benefit from that. There are some parameters that users must apply:

* For **hyperthreaded based jobs** users ***must*** specify the following options:

```bash
#SBATCH --ntasks-per-core=2                 # Mandatory for multithreaded jobs
#SBATCH --hint=multithread                  # Mandatory for multithreaded jobs
```

* For **non-hyperthreaded based jobs** users ***must*** specify the following options:
```bash
#SBATCH --ntasks-per-core=1                 # Mandatory for non-multithreaded jobs
#SBATCH --hint=nomultithread                # Mandatory for non-multithreaded jobs
```

### Shared nodes and exclusivity

The **Merlin5** and **Merlin6** clusters are designed in a way that should allow running MPI/OpenMP processes as well as single core based jobs. For allowing co-existence, nodes are configured by default in a shared mode. It means, that multiple jobs from multiple users may land in the same node. This behaviour can be changed by a user if they require exclusive usage of nodes.

By default, Slurm will try to allocate jobs on nodes that are already occupied by processes not requiring exclusive usage of a node. In this way, we fill up first mixed nodes and we ensure that free full resources are available for MPI/OpenMP jobs.

Exclusivity of a node can be setup by specific the ``--exclusive`` option as follows:

```bash
#SBATCH --exclusive
```

### Slurm CPU Recommended Settings

There are some settings that are not mandatory but would be needed or useful to specify. These are the following:

* ``--time``: mostly used when you need to specify longer runs in the ``general`` partition, also useful for specifying
shorter times. **This will affect scheduling priorities**, hence is important to define it (and to define it properly).

   ```bash
   #SBATCH --time=<D-HH:MM:SS>   # Time job needs to run
   ```

### Output and Errors

By default, Slurm script will generate standard output and errors files in the directory from where
you submit the batch script:

* standard output will be written into a file ``slurm-$SLURM_JOB_ID.out``.
* standard error will be written into a file ``slurm-$SLURM_JOB_ID.err``.

If you want to the default names it can be done with the options ``--output`` and ``--error``. In example:

```bash
#SBATCH --output=logs/myJob.%N.%j.out  # Generate an output file per hostname and jobid
#SBATCH --error=logs/myJob.%N.%j.err   # Generate an errori file per hostname and jobid
```

Use **man sbatch** (``man sbatch | grep -A36 '^filename pattern'``) for getting a list specification of **filename patterns**.

## CPU-based Jobs Settings

CPU-based jobs are available for all PSI users. Users must belong to the ``merlin6`` Slurm ``Account`` in order to be able
to run on CPU-based nodes. All users registered in Merlin6 are automatically included in the ``Account``.

### Slurm CPU Templates

The following examples apply to the **Merlin6** cluster.

#### Nomultithreaded jobs example

The following template should be used by any user submitting jobs to CPU nodes:

```bash
#!/bin/sh
#SBATCH --partition=<general|daily|hourly>  # Specify 'general' or 'daily' or 'hourly'
#SBATCH --time=<D-HH:MM:SS>                 # Strictly recommended when using 'general' partition.
#SBATCH --output=<output_file>              # Generate custom output file
#SBATCH --error=<error_file>                # Generate custom error  file
#SBATCH --ntasks-per-core=1                 # Mandatory for non-multithreaded jobs
#SBATCH --hint=nomultithread                # Mandatory for non-multithreaded jobs
##SBATCH --exclusive                        # Uncomment if you need exclusive node usage

## Advanced options example
##SBATCH --nodes=1                          # Uncomment and specify #nodes to use
##SBATCH --ntasks=44                        # Uncomment and specify #nodes to use
##SBATCH --ntasks-per-node=44               # Uncomment and specify #tasks per node
##SBATCH --cpus-per-task=44                 # Uncomment and specify the number of cores per task
```

#### Multithreaded jobs

The following template should be used by any user submitting jobs to CPU nodes:

```bash
#!/bin/sh
#SBATCH --partition=<general|daily|hourly>  # Specify 'general' or 'daily' or 'hourly'
#SBATCH --time=<D-HH:MM:SS>                 # Strictly recommended when using 'general' partition.
#SBATCH --output=<output_file>              # Generate custom output file
#SBATCH --error=<error_file>                # Generate custom error  file
#SBATCH --ntasks-per-core=2                 # Mandatory for multithreaded jobs
#SBATCH --hint=multithread                  # Mandatory for multithreaded jobs
##SBATCH --exclusive                        # Uncomment if you need exclusive node usage

## Advanced options example
##SBATCH --nodes=1                          # Uncomment and specify #nodes to use
##SBATCH --ntasks=88                        # Uncomment and specify #nodes to use
##SBATCH --ntasks-per-node=88               # Uncomment and specify #tasks per node
##SBATCH --cpus-per-task=88                 # Uncomment and specify the number of cores per task
```

## GPU-based Jobs Settings

**Merlin6** GPUs are available for all PSI users, however, this is restricted to any user belonging to the ``merlin-gpu`` account. By default, all users are added to this account (exceptions could apply).

### Merlin6 GPU account

When using GPUs, users must switch to the **merlin-gpu** Slurm account in order to be able to run on GPU-based nodes. This is done with the ``--account`` setting as follows:

```bash
#SBATCH --account=merlin-gpu     # The account 'merlin-gpu' must be used
```

### Slurm CPU Mandatory Settings

The following options are mandatory settings that **must be included** in your batch scripts:

```bash
#SBATCH --gres=gpu         # Always set at least this option when using GPUs
```

#### Slurm GPU Recommended Settings

GPUs are also a shared resource. Hence, multiple users can run jobs on a single node, but only one GPU per user process
must be used. Users can define which GPUs resources they need with the ``--gres`` option.
Valid ``gres`` options are: ``gpu[[:type]:count]`` where ``type=GTX1080|GTX1080Ti`` and ``count=<number of gpus to use>``
This would be according to the following rules:

In example:

```bash
#SBATCH --gres=gpu:GTX1080:4   # Use a node with 4 x GTX1080 GPUs
```

***Important note:*** Due to a bug in the configuration, ``[:type]`` (i.e. ``GTX1080`` or ``GTX1080Ti``) is not working. Users should skip that and use only ``gpu[:count]``. This will be fixed in the upcoming downtimes as it requires a full restart of the batch system.

### Slurm GPU Template

The following template should be used by any user submitting jobs to GPU nodes:

```bash
#!/bin/sh
#SBATCH --partition=gpu_<general|daily|hourly> # Specify 'general' or 'daily' or 'hourly'
#SBATCH --gres="gpu:<type>:<number_gpus>"   # You should specify at least 'gpu'
#SBATCH --time=<D-HH:MM:SS>                 # Strictly recommended when using 'general' partition.
#SBATCH --output=<output_file>              # Generate custom output file
#SBATCH --error=<error_file                 # Generate custom error  file
#SBATCH --ntasks-per-core=1                 # GPU nodes have hyper-threading disabled
#SBATCH --account=merlin-gpu                # The account 'merlin-gpu' must be used
##SBATCH --exclusive                        # Uncomment if you need exclusive node usage


## Advanced options example
##SBATCH --nodes=1                          # Uncomment and specify number of nodes to use
##SBATCH --ntasks=44                        # Uncomment and specify number of nodes to use
##SBATCH --ntasks-per-node=44               # Uncomment and specify number of tasks per node
##SBATCH --cpus-per-task=44                 # Uncomment and specify the number of cores per task
```


## Job status

The status of submitted jobs can be check with the `squeue` command:

```bash
$> squeue -u bliven_s
           JOBID PARTITION     NAME     USER ST       TIME  NODES NODELIST(REASON)
       134507729       gpu test_scr bliven_s PD       0:00      3 (AssocGrpNodeLimit)
       134507768   general test_scr bliven_s PD       0:00     19 (AssocGrpCpuLimit)
       134507729       gpu test_scr bliven_s PD       0:00      3 (Resources)
       134506301       gpu test_scr bliven_s PD       0:00      1 (Priority)
       134506288       gpu test_scr bliven_s  R       9:16      1 merlin-g-008
```

Common Statuses:

* **merlin-\***: Running on the specified host
* **(Priority)**: Waiting in the queue
* **(Resources)**: At the head of the queue, waiting for machines to become available
* **(AssocGrpCpuLimit), (AssocGrpNodeLimit)**: Job would exceed per-user limitations on
  the number of simultaneous CPUs/Nodes. Use `scancel` to remove the job and
  resubmit with fewer resources, or else wait for your other jobs to finish.
* **(PartitionNodeLimit)**: Exceeds all resources available on this partition.
  Run `scancel` and resubmit to a different partition (`-p`) or with fewer
  resources.

